/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.shape.preview;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import java.awt.BorderLayout;
import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import nl.cloudfarming.client.field.model.Shape;
import nl.cloudfarming.client.field.model.ShapeWidget;
import nl.cloudfarming.client.geoviewer.render.GeometricalWidget;
import nl.cloudfarming.client.geoviewer.render.SimpleGeoTranslator;
import nl.cloudfarming.client.field.shape.ShapeDataObject;
import org.netbeans.api.visual.model.ObjectScene;
import org.netbeans.api.visual.widget.LayerWidget;

/**
 * Panel which displays all the shapes within the provided ShapeDataObject
 *
 * @author Merijn Zengers
 */
public class ShapesPanel extends JPanel {

    private final JPanel contentPanel = new JPanel(new BorderLayout());
    private final JScrollPane jScrollPane;

    public ShapesPanel(ShapeDataObject shapeFile, ObjectScene scene, PreviewPanelPalette palette) {
        this.setLayout(new BorderLayout());
        
        this.setBackground(Color.white);

        jScrollPane = new javax.swing.JScrollPane();
        jScrollPane.setViewportView(contentPanel);
        this.add(jScrollPane);


        List<Geometry> geometries = new ArrayList();
        for (Shape shape : shapeFile.getShapes()) {
            geometries.add(shape.getGeometry());
        }

        LayerWidget mainLayer = new LayerWidget(scene);
        scene.setOpaque(false);
        scene.addChild(mainLayer);

        if (geometries.size() > 0) {
            Geometry[] g = geometries.toArray(new Geometry[geometries.size()]);
            GeometryCollection geoCollection = new GeometryCollection(g, g[0].getFactory());

            for (Shape shape : shapeFile.getShapes()) {
                GeometricalWidget widget = new ShapeWidget(scene, new SimpleGeoTranslator(), shape, palette, geoCollection.getEnvelope());

                widget.getActions().addAction(scene.createSelectAction());
                widget.getActions().addAction(scene.createObjectHoverAction());

                mainLayer.addChild(widget);
                scene.addObject(shape, widget);
            }
        }

        contentPanel.add(scene.createView());
    }
}
